﻿/*
    Name...........PencilFx
    Version.........1.0
    Description....Apply pencil sketch look effect
    Author..........Original Fx by Cuku / Scripting by llcheesell
*/


(function(){

    /*---------------------------------------------------------------------------------------------------------*/    
    function initGlobals(G)
    /*---------------------------------------------------------------------------------------------------------*/
    {                
        G.NAME                      =   "PencilFx.jsx";
        G.TITLE                     =   "PencilFx";
        //-----------------------------------------------------------------------------------------------------
        G.NO_PROJ_ERR               =   {en:"Open a project first.", jp:"プロジェクトを開いて下さい."};
        G.NO_COMP_ERR               =   {en:"Select a composition.", jp:"コンポジションを選択して下さい."};
        G.NO_LAYER_ERR              =   {en:"Select at least one layer.", jp:"少なくとも1つのレイヤーを選択して下さい."};
        G.INSTRUCTION              =   {en:"At first create the mask with Auto-trace dialog.\n\nIn the dialog make sure that uncheck Apply to new layer, and set Channel option to Alpha.\nIf you want to apply the effect to the footage with alpha chennel, the Time Span option should be set to Work Area.\nOtherwise Current Frame would be best.", jp:"はじめにオートトレースを行います。\n\n「新規レイヤーに適用」のチェックは必ず外し、チャンネルは「アルファ」に設定してください。\n「範囲」は、画面全体に適用する場合は「現在のフレーム」、動きのあるアルファ付きの素材に適用する場合は「ワークエリア」に設定します。"};
    }
    
    /*---------------------------------------------------------------------------------------------------------*/
    function loc(str)
    /*---------------------------------------------------------------------------------------------------------*/
    {
        return app.language == Language.JAPANESE ? str["jp"] : str["en"];
    }
    
    /*---------------------------------------------------------------------------------------------------------*/
    function throwMsg(msg)
    /*---------------------------------------------------------------------------------------------------------*/
    {
        alert(loc(msg), G.TITLE);
    }
    
    /*---------------------------------------------------------------------------------------------------------*/
    function PencilFx(comp)
    /*---------------------------------------------------------------------------------------------------------*/
    {
        
		throwMsg(G.INSTRUCTION);
        var activeLayer = comp.selectedLayers[0];
        activeLayer.duplicate();
        activeLayer.duplicate();
        activeLayer.duplicate();
        var n =activeLayer.index;
        var base = comp.layer(n);
        var scribble = comp.layer(n-1);
        var matte = comp.layer(n-2);
        var noise = comp.layer(n-3);
        
        base.name="base";
        scribble.name = "scribble";
        matte.name = "matte";
        noise.name = "noise";
        scribble.audioEnabled = false;
        matte.audioEnabled = false;
        noise.audioEnabled = false;
        base.selected = false;
        
        scribble.selected = true;
        app.executeCommand(3044);
        scribble.Effects.addProperty("ADBE Scribble Fill").name="Scribble";
        scribble.Effects.property("Scribble")(14).setValue(2);
        scribble.Effects.property("Scribble")(16).setValue(0.04);
        scribble.Effects.property("Scribble")(17).setValue(0.02);
        scribble.Effects.property("Scribble")(18).setValue(4);
        scribble.Effects.property("Scribble")(26).setValue(2);
        scribble.Effects.property("Scribble")(27).setValue(7);
        scribble.trackMatteType = TrackMatteType.LUMA
        
        matte.Effects.addProperty("ADBE Find Edges");
        matte.Effects.addProperty("ADBE HUE SATURATION").name="Saturation";
        matte.Effects.property("Saturation")(4).setValue(-100);
        matte.Effects.addProperty("ADBE Scatter").name="Scatter";
        matte.Effects.property("Scatter")(1).setValue(2);
        matte.Effects.addProperty("ADBE Gaussian Blur").name="GBlur";
        matte.Effects.property("GBlur")(1).setValue(0.4);
        matte.blendingMode = BlendingMode.SCREEN;
        
        noise.Effects.addProperty("ADBE Noise HLS Auto2").name="Noise HLS";
        noise.Effects.property("Noise HLS")(1).setValue(3);
        noise.Effects.property("Noise HLS")(3).setValue(22);
        noise.Effects.property("Noise HLS")(5).setValue(0.5);
        noise.Effects.property("Noise HLS")(6).setValue(0);
        noise.Effects.addProperty("ADBE Motion Blur").name="MBlur";
        noise.Effects.property("MBlur")(1).setValue(60);
        noise.Effects.property("MBlur")(2).setValue(4);
        noise.Effects.addProperty("ADBE Brightness & Contrast 2").name="BC";
        noise.Effects.property("BC")(1).setValue(50);
        noise.Effects.property("BC")(2).setValue(50);
        noise.Effects.addProperty("ADBE Sharpen").name="Sharpen";
        noise.Effects.property("Sharpen")(1).setValue(50);
        noise.blendingMode = BlendingMode.MULTIPLY;
        
        var turbulent = comp.layers.addSolid([1,1,1],"finish",comp.width,comp.height,comp.pixelAspect);
        turbulent.adjustmentLayer = true;
        turbulent.Effects.addProperty("ADBE Turbulent Displace").name="Turbulent";
        turbulent.Effects.property("Turbulent")(1).setValue(6);
        turbulent.Effects.property("Turbulent")(3).setValue(2);
        turbulent.Effects.property("Turbulent")(5).setValue(10);
        turbulent.Effects.property("Turbulent")(6).expression = '360*random();';
        turbulent.Effects.addProperty("ADBE Tint").name="Tint";
        turbulent.Effects.addProperty("ADBE Posterize Time").name="Posterize";
        turbulent.Effects.property("Posterize")(1).setValue(15);
        
        
        var precompLayers = new Array();
        precompLayers = [base.index,scribble.index,matte.index,noise.index,turbulent.index];
        comp.layers.precompose(precompLayers,"pencilFx");
        
        
        var pencilFx = comp.selectedLayers[0];
        pencilFx.name="pencilFx";
        pencilFx.Effects.addProperty("ADBE Slider Control").name = "Framerate";
        pencilFx.Effects.property("Framerate")(1).setValue(10);
        pencilFx.Effects.addProperty("ADBE Checkbox Control").name = "Stroke speed half";
        pencilFx.Effects.property("Stroke speed half")(1).setValue(true);
        pencilFx.Effects.addProperty("ADBE Slider Control").name = "Stroke width";
        pencilFx.Effects.property("Stroke width")(1).setValue(2);
        pencilFx.Effects.addProperty("ADBE Slider Control").name = "Stroke density";
        pencilFx.Effects.property("Stroke density")(1).setValue(4);
        pencilFx.Effects.addProperty("ADBE Slider Control").name = "Stroke brightness";
        pencilFx.Effects.property("Stroke brightness")(1).setValue(40);
        pencilFx.Effects.addProperty("ADBE Slider Control").name = "Stroke contrast";
        pencilFx.Effects.property("Stroke contrast")(1).setValue(0);
        pencilFx.Effects.addProperty("ADBE Slider Control").name = "Tint";
        pencilFx.Effects.property("Tint")(1).setValue(0);
        pencilFx.Effects.addProperty("ADBE Color Control").name = "Tint color";
        pencilFx.Effects.property("Tint color")(1).setValue([1,0,0]);
        pencilFx.Effects.addProperty("ADBE Slider Control").name = "Smear";
        pencilFx.Effects.property("Smear")(1).setValue(40);
        pencilFx.source.layer("scribble").Effects.property("Scribble")(14).expression = 'comp("'+comp.name+'").layer("pencilFx").effect("Stroke width")(1)';
        pencilFx.source.layer("scribble").Effects.property("Scribble")(18).expression = 'comp("'+comp.name+'").layer("pencilFx").effect("Stroke density")(1)';
        pencilFx.source.layer("scribble").Effects.property("Scribble")(27).expression = 'half=1;if (comp("'+comp.name+'").layer("pencilFx").effect("Stroke speed half")(1)==true) half=2;comp("'+comp.name+'").layer("pencilFx").effect("Framerate")(1)/half;';
        pencilFx.source.layer("noise").Effects.property("BC")(1).expression = 'comp("'+comp.name+'").layer("pencilFx").effect("Stroke brightness")(1)';
        pencilFx.source.layer("noise").Effects.property("BC")(2).expression = 'comp("'+comp.name+'").layer("pencilFx").effect("Stroke contrast")(1)';
        pencilFx.source.layer("finish").Effects.property("Turbulent")(2).expression = 'comp("'+comp.name+'").layer("pencilFx").effect("Smear")(1)';
        pencilFx.source.layer("finish").Effects.property("Tint")(1).expression = 'comp("'+comp.name+'").layer("pencilFx").effect("Tint color")(1)';
        pencilFx.source.layer("finish").Effects.property("Tint")(3).expression = 'comp("'+comp.name+'").layer("pencilFx").effect("Tint")(1)';
        pencilFx.source.layer("finish").Effects.property("Posterize")(1).expression = 'comp("'+comp.name+'").layer("pencilFx").effect("Framerate")(1)';

    }
    
    /*---------------------------------------------------------------------------------------------------------*/
    function main() 
    /*---------------------------------------------------------------------------------------------------------*/
    {
        // プロジェクトを開いているかチェック
        var proj = app.project;
        if (!proj) {
            throwMsg(G.NO_PROJ_ERR);
            return;
        }
        // コンポジションを選択してるかチェック
        var comp = proj.activeItem;
        if (!comp || !(comp instanceof CompItem)) {
            throwMsg(G.NO_COMP_ERR);
            return;
        }
        // レイヤーを選択する必要がある場合
        // レイヤーを少なくとも1つ選択してるかチェック
       if (comp.selectedLayers.length < 1){
           throwMsg(G.NO_LAYER_ERR);
           return;
       }
        
        // メイン処理であるtemplate()関数を実行
        app.beginUndoGroup(G.TITLE);
        PencilFx(comp);
        app.endUndoGroup();
    }
    
    /*---------------------------------------------------------------------------------------------------------*/
    // Entry Point
    /*---------------------------------------------------------------------------------------------------------*/
    
    var G = new Object();
    initGlobals(G);
    main();

})();
